@@ -1,5 +1,6 @@
 import os
 import re
+import sys
 import glob
 import json
 import time
@@ -7,6 +8,7 @@ import logging
 import threading
 import subprocess
 import six
+import base64
 from multiprocessing import Process, Queue
 try:
     from shlex import quote as cmd_quote
@@ -14,9 +16,10 @@ except ImportError:
     from pipes import quote as cmd_quote  # for Python 2.7
 from localstack import config
 from localstack.utils import bootstrap
+from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
     CaptureOutput, FuncThread, TMP_FILES, short_uid, save_file, rm_rf, in_docker,
-    to_str, run, cp_r, json_safe, get_free_tcp_port)
+    to_str, to_bytes, run, cp_r, json_safe, get_free_tcp_port)
 from localstack.services.install import INSTALL_PATH_LOCALSTACK_FAT_JAR
 from localstack.utils.aws.dead_letter_queue import lambda_error_to_dead_letter_queue, sqs_error_to_dead_letter_queue
 from localstack.utils.cloudwatch.cloudwatch_util import store_cloudwatch_logs, cloudwatched
@@ -58,11 +61,16 @@ LOG = logging.getLogger(__name__)
 # maximum time a pre-allocated container can sit idle before getting killed
 MAX_CONTAINER_IDLE_TIME_MS = 600 * 1000
 
+# SQS event source name
 EVENT_SOURCE_SQS = 'aws:sqs'
 
 # IP address of main Docker container (lazily initialized)
 DOCKER_MAIN_CONTAINER_IP = None
 
+# whether to use our custom Java executor, or the default from lambci
+# TODO: deprecated, should be removed in the future
+USE_CUSTOM_JAVA_EXECUTOR = False
+
 
 def get_from_event(event, key):
     try:
@@ -100,8 +108,9 @@ def get_main_endpoint_from_container():
                 DOCKER_MAIN_CONTAINER_IP = bootstrap.get_main_container_ip()
                 LOG.info('Determined main container target IP: %s' % DOCKER_MAIN_CONTAINER_IP)
         except Exception as e:
+            container_name = bootstrap.get_main_container_name()
             LOG.info('Unable to get IP address of main Docker container "%s": %s' %
-                (bootstrap.MAIN_CONTAINER_NAME, e))
+                (container_name, e))
     # return main container IP, or fall back to Docker host (bridge IP, or host DNS address)
     return DOCKER_MAIN_CONTAINER_IP or config.DOCKER_HOST_FROM_CONTAINER
 
@@ -112,6 +121,15 @@ class LambdaExecutor(object):
         # keeps track of each function arn and the last time it was invoked
         self.function_invoke_times = {}
 
+    def _prepare_environment(self, func_details):
+        # setup environment pre-defined variables for docker environment
+        result = func_details.envvars.copy()
+
+        # injecting aws credentials into docker environment if not provided
+        aws_stack.inject_test_credentials_into_env(result)
+
+        return result
+
     def execute(self, func_arn, func_details, event, context=None, version=None,
             asynchronous=False, callback=None):
         def do_execute(*args):
@@ -196,9 +214,7 @@ class LambdaExecutor(object):
 
 
 class ContainerInfo:
-    """
-    Contains basic information about a docker container.
-    """
+    """ Contains basic information about a docker container. """
     def __init__(self, name, entry_point):
         self.name = name
         self.entry_point = entry_point
@@ -224,7 +240,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         lambda_cwd = func_details.cwd
         runtime = func_details.runtime
         handler = func_details.handler
-        environment = func_details.envvars.copy()
+        environment = self._prepare_environment(func_details)
 
         # configure USE_SSL in environment
         if config.USE_SSL:
@@ -240,6 +256,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         main_endpoint = get_main_endpoint_from_container()
 
         environment['LOCALSTACK_HOSTNAME'] = main_endpoint
+        environment['EDGE_PORT'] = str(config.EDGE_PORT)
         environment['_HANDLER'] = handler
         if os.environ.get('HTTP_PROXY'):
             environment['HTTP_PROXY'] = os.environ['HTTP_PROXY']
@@ -249,13 +266,17 @@ class LambdaExecutorContainers(LambdaExecutor):
             environment['AWS_LAMBDA_FUNCTION_NAME'] = context.function_name
             environment['AWS_LAMBDA_FUNCTION_VERSION'] = context.function_version
             environment['AWS_LAMBDA_FUNCTION_INVOKED_ARN'] = context.invoked_function_arn
+            environment['AWS_LAMBDA_COGNITO_IDENTITY'] = json.dumps(context.cognito_identity or {})
+            if context.client_context is not None:
+                environment['AWS_LAMBDA_CLIENT_CONTEXT'] = json.dumps(to_str(
+                    base64.b64decode(to_bytes(context.client_context))))
 
         # custom command to execute in the container
         command = ''
         events_file = ''
 
-        # if running a Java Lambda, set up classpath arguments
-        if is_java_lambda(runtime):
+        if USE_CUSTOM_JAVA_EXECUTOR and is_java_lambda(runtime):
+            # if running a Java Lambda with our custom executor, set up classpath arguments
             java_opts = Util.get_java_opts()
             stdin = None
             # copy executor jar into temp directory
@@ -384,6 +405,9 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                 network = config.LAMBDA_DOCKER_NETWORK
                 network_str = '--network="%s"' % network if network else ''
 
+                dns = config.LAMBDA_DOCKER_DNS
+                dns_str = '--dns="%s"' % dns if dns else ''
+
                 mount_volume = not config.LAMBDA_REMOTE_DOCKER
                 lambda_cwd_on_host = Util.get_host_path_for_path_in_docker(lambda_cwd)
                 if (':' in lambda_cwd and '\\' in lambda_cwd):
@@ -402,10 +426,13 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                     ' -e AWS_LAMBDA_EVENT_BODY="$AWS_LAMBDA_EVENT_BODY"'
                     ' -e HOSTNAME="$HOSTNAME"'
                     ' -e LOCALSTACK_HOSTNAME="$LOCALSTACK_HOSTNAME"'
+                    ' -e EDGE_PORT="$EDGE_PORT"'
                     '  %s'  # env_vars
                     '  %s'  # network
+                    '  %s'  # dns
                     ' %s'
-                ) % (docker_cmd, rm_flag, container_name, mount_volume_str, env_vars_str, network_str, docker_image)
+                ) % (docker_cmd, rm_flag, container_name, mount_volume_str,
+                    env_vars_str, network_str, dns_str, docker_image)
                 LOG.debug(cmd)
                 run(cmd)
 
@@ -611,7 +638,6 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
     def __init__(self):
         super(LambdaExecutorSeparateContainers, self).__init__()
-        self.next_port = 1
         self.max_port = LAMBDA_API_UNIQUE_PORTS
         self.port_offset = LAMBDA_API_PORT_OFFSET
 
@@ -631,10 +657,12 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
         network = config.LAMBDA_DOCKER_NETWORK
         network_str = '--network="%s"' % network if network else ''
         if network == 'host':
-            port = str(self.next_port + self.port_offset)
+            port = get_free_tcp_port()
             env_vars['DOCKER_LAMBDA_API_PORT'] = port
             env_vars['DOCKER_LAMBDA_RUNTIME_PORT'] = port
-            self.next_port = (self.next_port + 1) % self.max_port
+
+        dns = config.LAMBDA_DOCKER_DNS
+        dns_str = '--dns="%s"' % dns if dns else ''
 
         env_vars_string = ' '.join(['-e {}="${}"'.format(k, k) for (k, v) in env_vars.items()])
         debug_docker_java_port = '-p {p}:{p}'.format(p=Util.debug_java_port) if Util.debug_java_port else ''
@@ -649,12 +677,14 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
                 ' %s'  # debug_docker_java_port
                 ' %s'  # env
                 ' %s'  # network
+                ' %s'  # dns
                 ' %s'  # --rm flag
                 ' %s %s'  # image and command
                 ')";'
                 '%s cp "%s/." "$CONTAINER_ID:/var/task"; '
                 '%s start -ai "$CONTAINER_ID";'
-            ) % (docker_cmd, entrypoint, debug_docker_java_port, env_vars_string, network_str, rm_flag,
+            ) % (docker_cmd, entrypoint, debug_docker_java_port,
+                env_vars_string, network_str, dns_str, rm_flag,
                  docker_image, command,
                  docker_cmd, lambda_cwd,
                  docker_cmd)
@@ -665,17 +695,18 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
                 ' %s -v "%s":/var/task'
                 ' %s'
                 ' %s'  # network
+                ' %s'  # dns
                 ' %s'  # --rm flag
                 ' %s %s'
             ) % (docker_cmd, entrypoint, lambda_cwd_on_host, env_vars_string,
-                 network_str, rm_flag, docker_image, command)
+                 network_str, dns_str, rm_flag, docker_image, command)
         return cmd
 
 
 class LambdaExecutorLocal(LambdaExecutor):
     def _execute(self, func_arn, func_details, event, context=None, version=None):
         lambda_cwd = func_details.cwd
-        environment = func_details.envvars.copy()
+        environment = self._prepare_environment(func_details)
 
         # execute the Lambda function in a forked sub-process, sync result via queue
         queue = Queue()
@@ -684,12 +715,17 @@ class LambdaExecutorLocal(LambdaExecutor):
 
         def do_execute():
             # now we're executing in the child process, safe to change CWD and ENV
-            if lambda_cwd:
-                os.chdir(lambda_cwd)
-            if environment:
-                os.environ.update(environment)
-            result = lambda_function(event, context)
-            queue.put(result)
+            path_before = sys.path
+            try:
+                if lambda_cwd:
+                    os.chdir(lambda_cwd)
+                    sys.path = [lambda_cwd] + sys.path
+                if environment:
+                    os.environ.update(environment)
+                result = lambda_function(event, context)
+                queue.put(result)
+            finally:
+                sys.path = path_before
 
         process = Process(target=do_execute)
         with CaptureOutput() as c:
@@ -735,9 +771,9 @@ class Util:
             opts = opts.replace('_debug_port_', ('%s' % cls.debug_java_port))
         else:
             # Parse the debug port from opts
-            m = re.match('.*address=(\\d+).*', opts)
+            m = re.match('.*address=(.+:)?(\\d+).*', opts)
             if m is not None:
-                cls.debug_java_port = m.groups()[0]
+                cls.debug_java_port = m.groups()[1]
 
         return opts
 
