@@ -18,10 +18,11 @@ from six.moves.socketserver import ThreadingMixIn
 from six.moves.urllib.parse import urlparse
 from six.moves.BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
 from localstack import config
-from localstack.config import USE_SSL, EXTRA_CORS_ALLOWED_HEADERS, EXTRA_CORS_EXPOSE_HEADERS
+from localstack.config import EXTRA_CORS_ALLOWED_HEADERS, EXTRA_CORS_EXPOSE_HEADERS
 from localstack.constants import APPLICATION_JSON
 from localstack.utils.server import http2_server
-from localstack.utils.common import FuncThread, generate_ssl_cert, to_bytes, json_safe, TMP_THREADS, path_from_url
+from localstack.utils.common import (
+    FuncThread, generate_ssl_cert, to_bytes, json_safe, TMP_THREADS, path_from_url, Mock)
 from localstack.utils.testutil import is_local_test_mode
 from localstack.utils.http_utils import uses_chunked_encoding, create_chunked_data
 from localstack.utils.aws.aws_responses import LambdaResponse
@@ -446,9 +447,13 @@ class DuplexSocket(ssl.SSLSocket):
             return peek_ssl_header()
         except Exception:
             # Fix for "[Errno 11] Resource temporarily unavailable" - This can
-            # happen if we're using a non-blocking socket in a blocking thread
+            #   happen if we're using a non-blocking socket in a blocking thread.
             newsock.setblocking(1)
-            return peek_ssl_header()
+            newsock.settimeout(1)
+            try:
+                return peek_ssl_header()
+            except Exception:
+                return False
 
 
 # set globally defined SSL socket implementation class
@@ -464,9 +469,10 @@ async def _accept_connection2(self, protocol_factory, conn, extra, sslcontext, *
 
 
 # patch asyncio server to accept SSL and non-SSL traffic over same port
-if hasattr(BaseSelectorEventLoop, '_accept_connection2'):
+if hasattr(BaseSelectorEventLoop, '_accept_connection2') and not hasattr(BaseSelectorEventLoop, '_ls_patched'):
     _accept_connection2_orig = BaseSelectorEventLoop._accept_connection2
     BaseSelectorEventLoop._accept_connection2 = _accept_connection2
+    BaseSelectorEventLoop._ls_patched = True
 
 
 class GenericProxy(FuncThread):
@@ -514,7 +520,7 @@ class GenericProxy(FuncThread):
 
     @classmethod
     def get_flask_ssl_context(cls, serial_number=None):
-        if USE_SSL:
+        if config.USE_SSL:
             _, cert_file_name, key_file_name = cls.create_ssl_cert(serial_number=serial_number)
             return (cert_file_name, key_file_name)
         return None
@@ -548,11 +554,8 @@ def run_proxy_server_http2(port, listener=None, forward_url=None, asynchronous=T
         method = request.method
         headers = request.headers
 
-        class T:
-            pass
-
-        request_handler = T()
-        request_handler.proxy = T()
+        request_handler = Mock()
+        request_handler.proxy = Mock()
         request_handler.proxy.port = port
         response = modify_and_forward(method=method, path=path_with_params, data_bytes=data, headers=headers,
             forward_base_url=forward_url, listeners=[listener], request_handler=None,
